﻿//DESCRIPTION: Apply styling in text variables

(function () {
	
	var headings;
	var hasStyles = false;
	var rhStyles = []; // Par styles used in running headers
	var cStyles = []; // Char styles used in headings
	var indd = app.documents.item(0);
	var none = app.activeDocument.characterStyles.item(0).name;
	var hide = getHideStyle();

	//-----------------------------------------------
	// Return a character style that hides text
	
	function getHideStyle () {
		var s = indd.characterStyles.item ('hide');
		if (!s.isValid) {
			s = indd.characterStyles.add ({
				name: 'hide',
				pointSize: 0.1,
				horizontalScale: 1,
				position: Position.SUPERSCRIPT,
			});
		}
		return s;
	}	
	
	//-----------------------------------------------
	// Return the paragraph styles used in 
	// running-header text variables

	function headingParagraphStyles () {
		var a = [];
		var v = app.activeDocument.textVariables.everyItem().getElements();
		for (var i = 0; i < v.length; i++) {
			if (v[i].variableType === VariableTypes.MATCH_PARAGRAPH_STYLE_TYPE) {
				a.push (v[i].variableOptions.appliedParagraphStyle);
			}
		}
		return a;
	}

	//----------------------------------------
	// Return a regular expression to test for paragraph-style names
	// used in running-header variables
	
	function getHeadingNames (pstyles) {
		var a = [];
		for (var i = 0; i < pstyles.length; i++) {
			a.push (pstyles[i].name);
		}
		return RegExp ('^(' + a.join('|') + ')$');
	}
	
	//----------------------------------------
	// Return stories on document pages with more than two paragraphs of text
	
	function getStories () {
		var a = [];
		var stories = indd.stories.everyItem().getElements();
		for (var i = 0; i < stories.length; i++) {
			if (stories[i].textContainers[0].parent instanceof Spread
				&& stories[i].storyType === StoryTypes.REGULAR_STORY
				&& stories[i].paragraphs.length > 2) {
					a.push (stories[i]);
			}
		}
		return a;
	}
	
	//----------------------------------------
	// Override a text frame if it hasn't
	// already been overridden

	function getRhFrame (page) {
		var r = page.textFrames.item ('Running header');
		if (r.isValid) {
			return r;
		}
		var m = page.masterPageItems;
		for (var i = 0; i < m.length; i++) {
			if (m[i].name === 'Running header') {
				return m[i].override (page);
			}
		}
		return null;
	}

	//-------------------------------------------
	// Insert the text tags into the section headings
	// Use the first letter of the character style's name
	// Add the character style to cStyles, we use them
	// to set up GREP styles

	function markRanges (par, r) {
		var cs, name;
		for (var i = r.length-1; i >= 0; i--) {
			name = r[i].appliedCharacterStyle.name;
			if (name != none) { // Skip (the equivalent of) [None]
				cStyles.push (r[i].appliedCharacterStyle); // cStyles is a global variable
				r[i].insertionPoints[-1].contents = '</' + name.slice(0,2) + '>';
				r[i].insertionPoints[0].contents = '<' + name.slice(0,2) + '>';
			}
		}
	}


	function process (story) {
		var ranges;
		var par = story.paragraphs.everyItem().getElements();
		
		for (var i = par.length-1; i >= 0; i--) {
			
			// Not a heading
			if (!headingNames.test (par[i].appliedParagraphStyle.name)) {
				continue;
			}

			// Insert the paragraph number and apply the hiding style
			
			if (par[i].textStyleRanges.length > 1) {
				markRanges (par[i], par[i].textStyleRanges);
				hasStyles = true;
			}
		
			if (par[i].bulletsAndNumberingListType === ListType.NUMBERED_LIST) {
				par[i].insertionPoints[0].appliedCharacterStyle = hide;
				par[i].insertionPoints[0].contents = par[i].bulletsAndNumberingResultText;
			}
		
		}
	}

	//----------------------------------------------
	
	function setGrepStyles () {
		var pStyles = headings.concat (rhStyles);
		var temp;
		
		function addGrepStyle (ps) {
			
			for (var i = 0; i < cStyles.length; i++) {
				temp = cStyles[i].name.slice (0,2);
				ps.nestedGrepStyles.add ({
					appliedCharacterStyle: cStyles[i],
					grepExpression: '<'+temp+'>.+?</'+temp+'>',
				});
			}
		}
		
		for (var i = 0; i < pStyles.length; i++) {
			addGrepStyle (pStyles[i]);
			pStyles[i].nestedGrepStyles.add ({
				appliedCharacterStyle: hide,
				grepExpression: '</?[\\u\\l]+?>',
			});
		}
	}
	
	//-------------------------------------------------------
	
	function overrideRunningHeaders () {
		var rh;
		var p = app.activeDocument.pages.everyItem().getElements();
		for (var i = 0; i < p.length; i++) {
			rh = getRhFrame (p[i]);
			if (!rh) {
				continue;
			}
			rhStyles.push (rh.paragraphs[0].appliedParagraphStyle);
			try {
				rh.textVariableInstances[0].convertToText();
			} catch (_) {
			}
		}
	}
	
	//================================================
	
	if (!app.activeDocument.textFrames.item ('Running header').isValid) {
		alert ('The document does not contain any \'Running header\' frames.');
		exit();
	}
	
	app.findGrepPreferences = null;
	
	headings = headingParagraphStyles();
	headingNames = getHeadingNames (headings);
	
	stories = getStories();

	for (var i = 0; i < stories.length; i++) {
		process (stories[i]);
	}

	// If there are just paragraph numbers,
	// no need to override rhs or set grep styles
	
	if (hasStyles) {
		overrideRunningHeaders();
		setGrepStyles();
	}

	indd.save (
		File (indd.filePath + '/' + 
			indd.name.replace (/\.indd$/, '-rh.indd'))
	);

}());